# Categories view

import time

from tabulate import tabulate

from ..models.base import get_session
from ..models.Category import CategoryModel
from ..models.Secret import SecretModel
from ..modules.misc import confirm, clear_screen
from . import menu


def all():
    """
        Return a list of all categories
    """

    return get_session().query(CategoryModel).filter(CategoryModel.active == 1).order_by(CategoryModel.id).all()


def to_table(rows=[]):
    """
        Transform rows in a table
    """

    # Retrieve id and name
    cats = [[cat.id, cat.name] for cat in rows]

    if len(cats) > 0:
        return tabulate(cats, headers=['Item', 'Category name'])
    else:
        return 'Empty!'


def pick(message='Select a category: ', optional=False):
    """
        Ask a user to pick a category
    """

    all_categories = all()

    if len(all_categories) == 0:
        print()
        print('You did not create any category yet. You can create one from the main menu.')
        return False

    # Display available categories
    print()
    print(to_table(all_categories))
    print()

    # Ask user input
    id_ = menu.get_input(message=message)

    # Cast as int
    try:
        id_int = int(id_)
        if id_int and exists(id_int):
            return id_int
    except ValueError:  # Not a valid number!
        pass

    # No category picked
    if optional and id_ == '':
        return None

    print()
    print('Invalid category number!')

    return False


def exists(id_):
    """
        Check if a category ID exists
    """

    if get_session().query(CategoryModel).filter(CategoryModel.id == int(id_)).filter(CategoryModel.active == 1).first():
        return True

    return False


def get_name(id_):
    """
        Get a category name from a category ID
    """

    if not id_:
        return ''

    cat = get_session().query(CategoryModel).filter(
        CategoryModel.id == int(id_)).filter(CategoryModel.active == 1).first()

    if cat:
        return cat.name

    return ''


def get_id(name):
    """
        Get a category ID from a category name
    """

    if not name:
        return None

    cat = get_session().query(CategoryModel).filter(
        CategoryModel.name == name).filter(CategoryModel.active == 1).first()

    if cat:
        return cat.id

    return None


def add(name):
    """
        Create a new category
    """

    cat = CategoryModel(name=name, active=1)
    get_session().add(cat)
    get_session().commit()

    return True


def add_input():
    """
        Ask user for a category name and create it
    """

    # Ask user input
    name = menu.get_input(message='Category name: ')

    # Return false if name is missing
    if not name:
        return False

    # Create category
    result = add(name=name)

    print()
    print('The category has been created.')

    time.sleep(2)

    return result


def rename(id_, new_name):
    """
        Rename a category
    """

    cat = get_session().query(CategoryModel).filter(
        CategoryModel.id == int(id_)).filter(CategoryModel.active == 1).first()

    if cat:
        cat.name = new_name
        get_session().add(cat)
        get_session().commit()

        return True

    return False


def rename_input():
    """
        Rename a category
    """

    # Get id
    id_ = pick()

    # Return false if id is invalid
    if not id_:
        return False

    # Ask user input
    name = menu.get_input(message='Category name: ')

    # Return false if name is missing
    if not name:
        return False

    result = rename(id_, name)

    if result is True:
        print()
        print('The category has been renamed.')

        time.sleep(2)

    return result


def delete(id_):
    """
        Disable a category
    """

    cat = get_session().query(CategoryModel).filter(
        CategoryModel.id == int(id_)).filter(CategoryModel.active == 1).first()

    if cat:
        cat.active = 0
        get_session().add(cat)
        get_session().commit()

        return True

    return False


def delete_input():
    """
        Delete a category
    """

    # Get id
    id_ = pick()

    # Return false if id is invalid
    if not id_:
        return False

    # Return false if the category is used
    if is_used(id_):
        print()
        print(
            'The category cannot be deleted because it is currently used by some secrets.')
        return False

    if confirm('Confirm deletion of the category "' + get_name(id_) + '"?', False):
        result = delete(id_)

        if result is True:
            print()
            print('The category has been deleted.')

            time.sleep(2)

        return result

    return False


def is_used(id_):
    """
        Check if a category ID is used by any secret
    """

    if get_session().query(SecretModel).filter(
            SecretModel.category_id == int(id_)).first():
        return True

    return False


def main_menu():
    """
        Categories menu
    """

    while True:
        # Clear screen
        clear_screen()

        # List categories
        print(to_table(all()))

        print()
        command = menu.get_input(
            message='Choose a command [(a)dd a category / (r)rename a category / (d)elete a category / (b)ack to Vault]: ',
            lowercase=True,
            non_locking_values=['l', 'q']
        )

        if command is False:
            print()

        # Action based on command
        if command == 'a':  # Add a category
            add_input()
            return
        elif command == 'r':  # Rename a category
            rename_input()
            return
        elif command == 'd':  # Delete a category
            delete_input()
            return
        elif command == 'b':  # Back to vault menu
            return



from unittest.mock import patch
import uuid

from ..base import BaseTest
from ...models.Secret import SecretModel
from ...lib.Encryption import Encryption
from ...modules.carry import global_scope


class Test(BaseTest):

    def setUp(self):
        # Set secret vars
        self.name = 'Vault'
        self.url = 'https://github.com/gabfl/vault'
        self.login = 'gab'
        self.password = 'some_password'
        self.notes = 'some notes'

        # Create a secret
        secret = SecretModel(name=self.name,
                             url=self.url,
                             login=self.login,
                             password=self.password,
                             notes=self.notes)
        self.session.add(secret)
        self.session.commit()

    def test_get_by_name(self):
        secret = self.session.query(
            SecretModel).filter_by(name=self.name).first()
        self.assertEqual(secret.name, self.name)
        self.assertEqual(secret.url, self.url)

    def test_repr(self):
        secret = self.session.query(SecretModel).get(1)
        print(secret)  # Required for codecov
        self.assertIsInstance(secret, object)

    def test_get_enc(self):
        secret = self.session.query(
            SecretModel).filter_by(name=self.name).first()
        self.assertIsInstance(secret.get_enc(), Encryption)

    def test_get_enc_2(self):
        with patch.dict(global_scope, {'enc': None}):
            secret = self.session.query(
                SecretModel).filter_by(name=self.name).first()
            self.assertRaises(RuntimeError, secret.get_enc)

    def test_getter_salt(self):
        secret = self.session.query(
            SecretModel).filter_by(name=self.name).first()
        self.assertIsInstance(secret.salt, bytes)

    def test_getter_password(self):
        secret = self.session.query(
            SecretModel).filter_by(name=self.name).first()
        self.assertEqual(secret.password, self.password)

    def test_getter_notes(self):
        secret = self.session.query(
            SecretModel).filter_by(name=self.name).first()
        self.assertEqual(secret.notes, self.notes)



from unittest.mock import patch
import tempfile

from sqlalchemy import engine
from sqlalchemy.orm import Session

from ..base import BaseTest
from ...models import base
from ...modules.carry import global_scope


class Test(BaseTest):

    def test_get_session(self):
        self.assertIsInstance(base.get_session(), Session)

    def test_get_session_2(self):
        # Text exception with `db_file` is not defined
        with patch.dict(global_scope, {'db_file': None}):
            self.assertRaises(RuntimeError, base.get_session)

    def test_drop_sessions(self):
        self.assertTrue(base.drop_sessions())
        self.assertEqual(base.sessions, {})

    def test_get_engine(self):
        self.assertIsInstance(base.get_engine(), engine.base.Engine)

    def test_get_engine_2(self):
        # Text exception with `db_file` is not defined
        with patch.dict(global_scope, {'db_file': None}):
            self.assertRaises(RuntimeError, base.get_engine)

    def test_get_engine_3(self):
        # Test non-encrypted connection with a temporary file
        file_ = tempfile.NamedTemporaryFile()
        with patch.dict(global_scope, {'db_file': file_.name}):
            self.assertIsInstance(base.get_engine(False), engine.base.Engine)

    def test_get_db_key(self):
        self.assertIsInstance(base.get_db_key(), str)

    def test_get_db_key_2(self):
        # Text exception with `enc` is not defined
        with patch.dict(global_scope, {'enc': None}):
            self.assertRaises(RuntimeError, base.get_db_key)

    def test_get_db_key_3(self):
        # Text exception with `conf` is not defined
        with patch.dict(global_scope, {'conf': None}):
            self.assertRaises(RuntimeError, base.get_db_key)

    def test_get_slashes(self):
        with patch.dict(global_scope, {'db_file': '/foo/bar'}):
            self.assertEqual(base.get_slashes(encrypted=True), '//')

    def test_get_slashes_2(self):
        with patch.dict(global_scope, {'db_file': 'foo/bar'}):
            self.assertEqual(base.get_slashes(encrypted=True), '/')

    def test_get_slashes_3(self):
        with patch.dict(global_scope, {'db_file': '/foo/bar'}):
            self.assertEqual(base.get_slashes(encrypted=False), '////')

    def test_get_slashes_4(self):
        with patch.dict(global_scope, {'db_file': 'foo/bar'}):
            self.assertEqual(base.get_slashes(encrypted=False), '///')



from unittest.mock import patch

from ..base import BaseTest
from ...models.Secret import SecretModel
from ...views import menu
from ...modules.carry import global_scope


class Test(BaseTest):

    def setUp(self):
        # Preserve enc to restore it on tear down
        self.enc_save = global_scope['enc']

    def tearDown(self):
        # restore enc in global scope
        global_scope['enc'] = self.enc_save

    def test_get_input(self):
        with patch('builtins.input', return_value='some input'):
            self.assertEqual(menu.get_input(), 'some input')

    def test_get_input_2(self):
        with patch('getpass.getpass', return_value='some secure input'):
            self.assertEqual(menu.get_input(secure=True), 'some secure input')

    def test_get_input_3(self):
        with patch('builtins.input', return_value='SOME INPUT'):
            self.assertEqual(menu.get_input(lowercase=True), 'some input')

    def test_get_input_4(self):
        with patch('builtins.input', return_value='some input'):
            self.assertEqual(menu.get_input(
                non_locking_values=['a', 'b', 'c']), 'some input')

    def test_unlock(self):
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(menu.unlock(redirect_to_menu=False))

    @patch.object(menu, 'menu')
    def test_unlock_2(self, patched):
        patched.return_value = None
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertIsNone(menu.unlock(redirect_to_menu=True))

    def test_unlock_3(self):
        with patch('getpass.getpass', return_value='wrong password'):
            self.assertRaises(SystemExit, menu.unlock)

    def test_unlock_4(self):
        # Simulate user pressing Ctrl-C
        with patch('getpass.getpass', return_value=False):
            self.assertRaises(SystemExit, menu.unlock)

    def test_validate_key(self):
        self.assertTrue(menu.validate_key(
            self.secret_key))

    def test_validate_key_2(self):
        self.assertFalse(menu.validate_key(
            'some invalid key'))

    def test_menu(self):
        with patch('builtins.input', return_value='q'):
            self.assertRaises(SystemExit, menu.menu)

    def test_menu_2(self):
        self.assertRaises(SystemExit, menu.menu, next_command='q')

    @patch.object(menu, 'unlock')
    def test_lock(self, patched):
        patched.return_value = None
        self.assertIsNone(menu.lock())
        self.assertIsNone(global_scope['enc'])

    def test_quit(self):
        self.assertRaises(SystemExit, menu.quit)

    def test_set_autolock_timer(self):
        menu.set_autolock_timer()
        self.assertIsInstance(menu.timer, int)

    def test_check_autolock_timer(self):
        menu.check_autolock_timer()
        self.assertIsNone(menu.check_autolock_timer())

    # @patch.object(menu, 'menu')
    # def test_check_autolock_timer_2(self, patched):
    #     patched.return_value = None
    #     with patch('getpass.getpass', return_value=self.secret_key):
    #         menu.timer = 100
    #         self.assertIsNone(menu.check_autolock_timer())

    def test_check_then_set_autolock_timer(self):
        menu.check_then_set_autolock_timer()
        self.assertIsNone(menu.check_then_set_autolock_timer())



from ..base import BaseTest
from ...models.Category import CategoryModel


class Test(BaseTest):

    def setUp(self):
        # Create category
        category = CategoryModel(name='My category', active=1)
        self.session.add(category)
        self.session.commit()

    def test_get_by_name(self):
        category = self.session.query(
            CategoryModel).filter_by(name='My category').first()
        self.assertEqual(category.name, 'My category')
        self.assertEqual(category.active, 1)

    def test_repr(self):
        category = self.session.query(CategoryModel).get(1)
        print(category)  # Required for codecov
        self.assertIsInstance(category, object)



from sqlalchemy import Column, Integer, String

from .base import Base


class CategoryModel(Base):
    __tablename__ = 'categories'

    id = Column(Integer, primary_key=True)
    name = Column(String)
    active = Column(Integer, default=1)

    def __repr__(self):
        return "<CategoryModel(id='%s', name='%s', active='%d')>" % (
            self.id, self.name, self.active)



import hashlib
import time

import pyperclip

from ..modules.carry import global_scope

clipboard_signature = None


def copy(to_copy, name='password', erase=False):
    """
        Copy an item to the clipboard
    """

    global clipboard_signature

    # Discard invalid input like `None` or empty strings
    if not erase and (type(to_copy) is not str or to_copy == ''):
        print('* Nothing to copy!')
        return False

    try:
        pyperclip.copy(to_copy)
    except pyperclip.PyperclipException:
        print('* Error: could not find a copy/paste mechanism for your system')
        return False

    if not erase:
        print('* The %s has been copied to the clipboard.' % (name))

    # Save signature
    clipboard_signature = get_signature(to_copy)


def is_changed():
    """
        Returns `True` if the clipboard content has changed
    """

    if clipboard_signature:
        return clipboard_signature != get_signature(pyperclip.paste())


def get_signature(item):
    """
        Returns the sha256 hash of a string
    """

    hash_object = hashlib.sha256(item.encode())
    return hash_object.hexdigest()


def wait():
    """
        Wait X seconds and erase the clipboard
    """

    if not clipboard_signature:
        return None

    print("* Clipboard will be erased in %s seconds" %
          (global_scope['conf'].clipboardTTL))

    try:
        # Loop until the delay is elapsed
        for i in range(0, int(global_scope['conf'].clipboardTTL)):
            print('.', end='', flush=True)
            time.sleep(1)  # Sleep 1 sec

            # Stop timer if clipboard content has changed
            if is_changed():
                break
    except KeyboardInterrupt as e:
        # Will catch `^-c` and immediately erase the clipboard
        pass

    print()
    erase()


def erase():
    """
        Erase clipboard content
    """

    global clipboard_signature

    if not is_changed():  # We will not empty the clipboard if its content has changed
        copy(to_copy='', erase=True)  # Empty clipboard
    clipboard_signature = ''  # Reset clipboard signature



from unittest.mock import patch

from ..base import BaseTest
from ...models.Category import CategoryModel
from ...models.Secret import SecretModel
from ...views import categories


class Test(BaseTest):

    def setUp(self):
        # Create some categories
        category_1 = CategoryModel(name='My category 1')
        self.session.add(category_1)
        category_2 = CategoryModel(name='My category 2')
        self.session.add(category_2)
        category_3 = CategoryModel(name='My category 3')
        self.session.add(category_3)
        category_4 = CategoryModel(name='My disabled category', active=0)
        self.session.add(category_4)
        self.session.commit()

    def tearDown(self):
        self.session.query(CategoryModel).delete()
        self.session.commit()

    def test_all(self):
        cats = categories.all()
        self.assertIsInstance(cats, list)
        self.assertEqual(len(cats), 3)

    def test_to_table(self):
        self.assertIsInstance(categories.to_table(categories.all()), str)

    def test_to_table_2(self):
        self.assertEqual(categories.to_table([]), 'Empty!')

    def test_pick(self):
        with patch('builtins.input', return_value='1'):
            self.assertEqual(categories.pick(), 1)

    def test_pick_2(self):
        with patch('builtins.input', return_value='1234'):
            self.assertFalse(categories.pick())

    def test_pick_3(self):
        with patch('builtins.input', return_value=''):
            self.assertFalse(categories.pick())

    def test_pick_4(self):
        with patch('builtins.input', return_value=''):
            self.assertIsNone(categories.pick(optional=True))

    @patch.object(categories, 'all')
    def test_pick_5(self, patched):
        patched.return_value = []

        # Test when there are no categories (all() returns `[]`)
        with patch('builtins.input', return_value=''):
            self.assertFalse(categories.rename_input())

    def test_exists(self):
        self.assertTrue(categories.exists(1))
        self.assertTrue(categories.exists('1'))

    def test_exists_2(self):
        self.assertFalse(categories.exists(1234))
        self.assertFalse(categories.exists('1234'))

    def test_get_name(self):
        self.assertEqual(categories.get_name(1), 'My category 1')
        self.assertEqual(categories.get_name('1'), 'My category 1')

    def test_get_name_2(self):
        self.assertEqual(categories.get_name(1234), '')
        self.assertEqual(categories.get_name('1234'), '')

    def test_get_name_3(self):
        self.assertEqual(categories.get_name(None), '')

    def test_get_id(self):
        self.assertEqual(categories.get_id('My category 1'), 1)

    def test_get_id_2(self):
        self.assertIsNone(categories.get_id('Some invalid name'))

    def test_get_id_3(self):
        self.assertIsNone(categories.get_id(None))

    def test_add(self):
        self.assertTrue(categories.add('My new category'))

        cat = self.session.query(CategoryModel).filter(
            CategoryModel.name == 'My new category').first()
        self.assertEqual(cat.name, 'My new category')

    def test_add_input(self):
        with patch('builtins.input', return_value='My new category'):
            self.assertTrue(categories.add_input())

    def test_add_input_2(self):
        with patch('builtins.input', return_value=None):
            self.assertFalse(categories.add_input())

    def test_rename(self):
        # Create a category
        categories.add('My new category')
        cat = self.session.query(CategoryModel).filter(
            CategoryModel.name == 'My new category').first()

        # Rename it
        self.assertTrue(categories.rename(cat.id, 'Some new name'))
        self.assertEqual(categories.get_name(cat.id), 'Some new name')

    @patch.object(categories, 'pick')
    def test_rename_input(self, patched):
        patched.return_value = 1

        with patch('builtins.input', return_value='new name'):
            self.assertTrue(categories.rename_input())

    @patch.object(categories, 'pick')
    def test_rename_input_2(self, patched):
        patched.return_value = 1

        with patch('builtins.input', return_value=''):
            self.assertFalse(categories.rename_input())

    @patch.object(categories, 'pick')
    def test_rename_input_3(self, patched):
        patched.return_value = None

        with patch('builtins.input', return_value=''):
            self.assertFalse(categories.rename_input())

    @patch.object(categories, 'pick')
    def test_rename_input_4(self, patched):
        patched.return_value = 1234

        with patch('builtins.input', return_value='new name'):
            self.assertFalse(categories.rename_input())

    def test_rename_2(self):
        # Test with non existent category
        self.assertFalse(categories.rename(1234, 'Some new name'))

    def test_delete(self):
        # Create a category
        categories.add('My new category')
        cat = self.session.query(CategoryModel).filter(
            CategoryModel.name == 'My new category').first()

        # Rename it
        self.assertTrue(categories.delete(cat.id))
        self.assertFalse(categories.exists(cat.id))

        # Deleting it again should fail
        self.assertFalse(categories.delete(cat.id))

    def test_delete_2(self):
        # Test with non existent category
        self.assertFalse(categories.rename(1234, 'Some new name'))

    @patch.object(categories, 'pick')
    def test_delete_input(self, patched):
        # Successful deletion
        patched.return_value = 1

        with patch('builtins.input', return_value='y'):
            self.assertTrue(categories.delete_input())

    @patch.object(categories, 'pick')
    def test_delete_input_2(self, patched):
        # Confirmation refused
        patched.return_value = 1

        with patch('builtins.input', return_value='n'):
            self.assertFalse(categories.delete_input())

    @patch.object(categories, 'pick')
    def test_delete_input_3(self, patched):
        # Invalid category number
        patched.return_value = 1234

        with patch('builtins.input', return_value='y'):
            self.assertFalse(categories.delete_input())

    @patch.object(categories, 'pick')
    def test_delete_input_4(self, patched):
        # No input
        patched.return_value = ''

        self.assertFalse(categories.delete_input())

    @patch.object(categories, 'pick')
    def test_delete_input_5(self, patched):
        # It should not be possible to delete a category currently used
        patched.return_value = '1'

        # Create a secret
        secret = SecretModel(name='Name', url='-', login='login',
                             password='password', category_id=1)
        self.session.add(secret)
        self.session.commit()

        self.assertFalse(categories.delete_input())

    def test_is_used(self):
        # Create a secret
        secret = SecretModel(name='Name', url='-', login='login',
                             password='password', category_id=1)
        self.session.add(secret)
        self.session.commit()

        self.assertTrue(categories.is_used(1))
        self.assertFalse(categories.is_used(1234))

    @patch.object(categories, 'add_input')
    def test_main_menu(self, patched):
        patched.return_value = None

        with patch('builtins.input', return_value='a'):
            self.assertIsNone(categories.main_menu())

    @patch.object(categories, 'rename_input')
    def test_main_menu_2(self, patched):
        patched.return_value = None

        with patch('builtins.input', return_value='r'):
            self.assertIsNone(categories.main_menu())

    @patch.object(categories, 'delete_input')
    def test_main_menu_3(self, patched):
        patched.return_value = None

        with patch('builtins.input', return_value='d'):
            self.assertIsNone(categories.main_menu())

    def test_main_menu_4(self):
        with patch('builtins.input', return_value='b'):
            self.assertIsNone(categories.main_menu())



from unittest.mock import patch
import tempfile
import json

from ..base import BaseTest
from ...views import import_export
from ...models.Secret import SecretModel
from ...models.Category import CategoryModel
from ...modules.carry import global_scope


class Test(BaseTest):

    def setUp(self):
        # Create some secrets
        secret_1 = SecretModel(name='Paypal',
                               url='https://www.paypal.com',
                               login='gab@gmail.com',
                               password='password123',
                               notes='Some notes',
                               category_id=1)
        self.session.add(secret_1)
        secret_2 = SecretModel(name='Gmail',
                               url='https://www.gmail.com',
                               login='gab@gmail.com',
                               password='password;123',
                               notes='Some notes\nsome more notes')
        self.session.add(secret_2)
        secret_3 = SecretModel(name='eBay',
                               url='https://www.ebay.com',
                               login='gab@gmail.com',
                               password='123password',
                               notes='Some notes')
        self.session.add(secret_3)

        # Add a category as well
        category_1 = CategoryModel(name='My category 1')
        self.session.add(category_1)

        self.session.commit()

    def test_import_(self):
        with patch('builtins.input', return_value='y'):
            with patch('getpass.getpass', return_value=self.secret_key):
                self.assertTrue(import_export.import_(
                    format_='json',
                    path='sample/export.json'))

    def test_import_2(self):
        self.assertRaises(ValueError, import_export.import_,
                          format_='some_invalid_format', path='/tmp/')

    def test_export_(self):
        # Create a temporary file
        file_ = tempfile.NamedTemporaryFile(delete=False)

        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(import_export.export_(
                format_='json', path=file_.name))

    def test_export_2(self):
        self.assertRaises(ValueError, import_export.export_,
                          format_='some_invalid_format', path='/tmp/')

    def test_export_to_json(self):
        # Create a temporary file
        file_ = tempfile.NamedTemporaryFile(delete=False)

        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(import_export.export_to_json(file_.name))

        # Try read the file
        with open(file_.name, mode='r') as f:
            # Get content
            content = f.read()

            # Decode content
            content = json.loads(content)

            # The content should be a list
            self.assertIsInstance(content, list)

            # Each item should be a dict
            for item in content:
                self.assertIsInstance(item, dict)

    def test_import_from_json(self):
        # Test basic import
        global_scope['enc'] = None
        with patch('builtins.input', return_value='y'):
            with patch('getpass.getpass', return_value=self.secret_key):
                self.assertTrue(import_export.import_from_json(
                    'sample/export.json'))

    def test_import_from_json_2(self):
        # Test import when vault is already unlocked (in unit test base)
        with patch('builtins.input', return_value='y'):
            self.assertTrue(import_export.import_from_json(
                'sample/export.json'))

    def test_import_from_json_3(self):
        # Test import with confirmation denied
        with patch('builtins.input', return_value='n'):
            with patch('getpass.getpass', return_value=self.secret_key):
                self.assertFalse(import_export.import_from_json(
                    'sample/export.json'))

    def test_to_table(self):
        self.assertIsInstance(import_export.to_table(
            [['name', 'url', 'login', 'password', 'notes', 'category']]), str)

    def test_to_table_2(self):
        self.assertIsInstance(import_export.to_table([]), str)
        self.assertEqual(import_export.to_table([]), 'Empty!')

    def test_read_file(self):
        # Write a temporary file
        file_ = tempfile.NamedTemporaryFile(delete=False)
        file_.write(b'Some file content')
        file_.close()

        self.assertEqual(import_export.read_file(
            file_.name), 'Some file content')

    def test_read_file_2(self):
        # Create a temporary directory
        dir_ = tempfile.TemporaryDirectory()

        self.assertRaises(SystemExit, import_export.read_file,
                          dir_.name + '/non/existent')

        # Cleanup dir
        dir_.cleanup()

    def test_save_file(self):
        # Create a temporary directory
        dir_ = tempfile.TemporaryDirectory()

        self.assertTrue(import_export.save_file(
            dir_.name + '/file', 'some content'))

        # Cleanup dir
        dir_.cleanup()

    def test_save_file_2(self):
        # Create a temporary directory
        dir_ = tempfile.TemporaryDirectory()

        self.assertFalse(import_export.save_file(
            dir_.name + '/non/existent/file', 'some content'))

        # Cleanup dir
        dir_.cleanup()

    def test_unlock(self):
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(import_export.unlock())



import tempfile
from pathlib import Path
from unittest.mock import patch
import os
import uuid

from .base import BaseTest
from .. import vault
from ..modules import misc
from ..modules.carry import global_scope
from ..lib.Config import Config
from ..views import menu, setup


class Test(BaseTest):

    def test_get_vault_path(self):
        self.assertEqual(vault.get_vault_path('/some/path'), '/some/path')

    def test_get_vault_path_2(self):
        self.assertEqual(vault.get_vault_path(),
                         os.path.expanduser('~') + '/.vault/.secure.db')

    def test_get_config_path(self):
        self.assertEqual(vault.get_config_path('/some/path'), '/some/path')

    def test_get_config_path_2(self):
        self.assertEqual(vault.get_config_path(),
                         os.path.expanduser('~') + '/.vault/.config')

    def test_check_directory(self):
        self.assertIsInstance(vault.check_directory(
            os.path.expanduser('~') + '/.vault/.secure.db', os.path.expanduser('~') + '/.vault/.config'), bool)

    def test_check_directory_2(self):
        # Set temporary files
        file_config = tempfile.NamedTemporaryFile()
        file_vault = tempfile.NamedTemporaryFile()

        self.assertIsNone(vault.check_directory(
            file_vault.name, file_config.name))

    def test_config_update(self):
        self.assertTrue(vault.config_update(clipboard_TTL='5'))

    def test_config_update_2(self):
        self.assertTrue(vault.config_update(auto_lock_TTL='5'))

    def test_config_update_3(self):
        self.assertTrue(vault.config_update(hide_secret_TTL='5'))

    @patch.object(menu, 'menu')
    def test_initialize(self, patched):
        # Test unlock

        patched.return_value = None

        # Try to unlock with the master key previously chosen
        with patch('getpass.getpass', return_value=self.secret_key):
            vault.initialize(
                global_scope['db_file'], self.conf_path.name + '/config')

    @patch.object(vault, 'erase_vault')
    def test_initialize_2(self, patched):
        # Test erase

        patched.return_value = 'patched'

        # Set temporary files
        file_vault = tempfile.NamedTemporaryFile(delete=False)
        Path(file_vault.name).touch()

        self.assertRaises(SystemExit, vault.initialize,
                          file_vault.name, self.conf_path.name + '/config', erase=True)

    def test_initialize_3(self):
        # Test re-keyi

        # Set temporary files
        file_vault = tempfile.NamedTemporaryFile(delete=False)
        Path(file_vault.name).touch()

        self.assertRaises(SystemExit, vault.initialize,
                          file_vault.name, self.conf_path.name + '/config', rekey_vault=True)

    @patch.object(vault, 'import_')
    def test_initialize_4(self, patched):
        # Test import

        patched.return_value = 'patched'

        # Set temporary files
        file_vault = tempfile.NamedTemporaryFile(delete=False)
        Path(file_vault.name).touch()

        self.assertRaises(SystemExit, vault.initialize,
                          file_vault.name, self.conf_path.name + '/config', import_items=True)

    @patch.object(vault, 'export_')
    def test_initialize_5(self, patched):
        # Test export

        patched.return_value = 'patched'

        # Set temporary files
        file_vault = tempfile.NamedTemporaryFile(delete=False)
        Path(file_vault.name).touch()

        self.assertRaises(SystemExit, vault.initialize,
                          file_vault.name, self.conf_path.name + '/config', export=True)

    @patch.object(vault.setup, 'initialize')
    @patch.object(vault, 'unlock')
    def test_initialize_6(self, patched, patched2):
        # Test export

        patched.return_value = 'patched'
        patched2.return_value = 'patched'

        # Set temporary files
        dir_ = tempfile.TemporaryDirectory()

        self.assertIsNone(vault.initialize(dir_.name + '/some/new/file',
                                           self.conf_path.name + '/config'))
